from django.http.response import HttpResponse


class APIMixinView:
    """
    APIView adalah class view untuk membuat Website API
    Cara kerjanya adalah dengan menggunakan variabel GET untuk menerima data.

    Class ini tidak bisa digunakan sendiri.
    Class ini harus menjadi mixin Class View karena perlu trigger untuk
    memanggil method get().

    ```py
    class ExampleAPIView(APIMixinView, View):
        pass
    ```
    """

    APIDict = {}

    def get(self, *args, **kwargs):
        self.get_data(*args, **kwargs)
        if self.process_data(*args, **kwargs):
            return self.success(*args, **kwargs)
        return self.failure(*args, **kwargs)

    def get_data(self, *args, **kwargs):
        iterator = self.APIDict or self.request.GET
        for k, v in iterator.items():
            self.APIDict[k] = self.request.GET.get(k, v)

    def process_data(self, *args, **kwargs):
        return True

    def success(self, *args, **kwargs):
        return HttpResponse(status=200)

    def failure(self, *args, **kwargs):
        return HttpResponse(status=500)



def get_class_method(cls):
    """
    Mengembalikan berupa tuple yg berisi list dari method dalam class

    ```python
    class ExampleGetClassMethod:
        def a():
            return [x for x in range(10)]

        def b():
            return [x for x in range(10)]

        def c():
            return [x for x in range(10)]

        def d():
            return [x for x in range(10)]

    print(get_class_method(ExampleGetClassMethod))
    ```
    """
    for x in dir(cls):
        a = getattr(cls, x)
        if not x.startswith("__") and callable(a):
            yield a



from ..iconsole.console_run import console_run


def github_pull():
    """
    Menjalankan command `git pull`

    ```py
    github_pull()
    ```
    """
    console_run("Git Pull", "git pull")


if __name__ == "__main__":
    github_pull()



import random
import string


def password_generator(
    length=8, characters=string.ascii_letters + string.digits + string.punctuation
):
    """
    Membuat pssword secara acak

    ```python
    print(password_generator())
    ```
    """

    password = ""
    for _ in range(length):
        password += random.choice(characters)

    return password


if __name__ == "__main__":
    from ..ifunctions.iargv import iargv

    length = iargv(1, int, 8)
    print(password_generator(length))



from .PintUregQuantity import PintUregQuantity


def calculate(teks):
    """
    Mengembalikan hasil dari perhitungan teks menggunakan modul pint.
    Mendukung perhitungan matematika dasar dengan satuan.

    Return value:
    - Berupa class Quantity dari modul pint

    Format:
    - f"{result:~P}"            -> pretty
    - f"{result:~H}"            -> html
    - result.to_base_units()    -> SI
    - result.to_compact()       -> human readable

    ```python
    fx = "3 meter * 10 cm * 3 km"
    res = calculate(fx)
    print(res)
    print(res.to_base_units())
    print(res.to_compact())
    print(f"{res:~P}")
    print(f"{res:~H}")
    ```
    """
    return PintUregQuantity(teks)


if __name__ == "__main__":
    from ..ifunctions.iargv import iargv

    if i := iargv(1):
        print(i)



import lxml


def idumps_html(data, indent=None):
    """
    Serialisasi python variabel menjadi HTML.
    ```
    List -> <ul>...</ul>
    Dict -> <table>...</table>
    ```

    ```python
    data = {
        'abc': 123,
        'list': [1, 2, 3, 4, 5],
        'dict': {'a': 1, 'b':2, 'c':3},
    }
    print(idumps_html(data))
    ```
    """

    def to_ul(data):
        ul = lxml.etree.Element("ul")
        for i in data:
            li = lxml.etree.SubElement(ul, "li")
            li.append(to_html(i))
        return ul

    def to_table(data: dict):
        table = lxml.etree.Element("table")
        tbody = lxml.etree.SubElement(table, "tbody")
        for i, v in data.items():
            tr = lxml.etree.SubElement(tbody, "tr")
            th = lxml.etree.SubElement(tr, "th")
            th.text = str(i)
            td = lxml.etree.SubElement(tr, "td")
            td.append(to_html(v))
        return table

    def to_text(data):
        span = lxml.etree.Element("span")
        span.text = str(data)
        return span

    def to_html(data):
        struct = {
            dict: to_table,
            list: to_ul,
            tuple: to_ul,
            set: to_ul,
        }
        return struct.get(type(data), to_text)(data)

    html = to_html(data)
    if indent:
        lxml.etree.indent(html, space=" " * indent)
    return lxml.etree.tostring(html, pretty_print=True, encoding="unicode")



import pypipr


def main():
    m = [x for x in dir(pypipr) if not x.startswith("__")]
    a = pypipr.iargv(1)
    while isinstance(m, list):
        if a is None:
            for i, v in dict(enumerate(m)).items():
                print(f"{i}. {v}")
            pypipr.print_colorize("Masukan Nomor Urut atau Nama Fungsi : ", text_end="")
            a = input()
            print()
            pypipr.exit_if_empty(len(a))

        if a.isdigit():
            m = m[int(a)]
        else:
            m = [v for v in m if v.__contains__(a)]
            pypipr.exit_if_empty(m)
            if len(m) == 1:
                m = m[0]
        a = None

    f = getattr(pypipr, m)

    pypipr.print_colorize(m)
    print(f.__doc__)

    import inspect
    s = inspect.signature(f)
    print(m, end="")
    pypipr.print_colorize(s)
    
    # a = []
    k = {}
    for i, v in s.parameters.items():
        # pypipr.print_dir(v)
        print(f"{i} [{v.default}] : ", end="")
        o = input()
        if len(o):
            # if isinstance(v.default, inspect._empty):
            #     a.append(o)
            # else:
            #     k[i] = o
            k[i] = eval(o)

    # print(a)
    print(k)
    # f(*a, **k)
    f(**k)




if __name__ == "__main__":
    main()



from .print_log import print_log


def log(text=None):
    """
    Decorator untuk mempermudah pembuatan log karena tidak perlu mengubah
    fungsi yg sudah ada.
    Melakukan print ke console untuk menginformasikan proses yg sedang
    berjalan didalam program.

    ```py
    @log
    def some_function():
        pass

    @log()
    def some_function_again():
        pass

    @log("Calling some function")
    def some_function_more():
        pass

    some_function()
    some_function_again()
    some_function_more()
    ```
    """

    def inner_log(func=None):
        def callable_func(*args, **kwargs):
            main_function(text)
            result = func(*args, **kwargs)
            return result

        def main_function(param):
            print_log(param)

        if func is None:
            return main_function(text)
        return callable_func

    if text is None:
        return inner_log
    elif callable(text):
        return inner_log(text)
    else:
        # inner_log(None)
        return inner_log



def print_to_last_line(text: str):
    """
    Melakukan print ke konsol tetapi akan menimpa baris terakhir.
    Berguna untuk memberikan progress secara interaktif.

    ```python
    c = input("masukkan apa saja : ")
    print_to_last_line(f"masukkan apa saja : {c} [ok]")
    ```
    """
    print(f"\033[F{text}")



from .ibuiltins.ComparePerformance import ComparePerformance
from .ibuiltins.LINUX import LINUX
from .ibuiltins.RunParallel import RunParallel
from .ibuiltins.WINDOWS import WINDOWS
from .ibuiltins.avg import avg
from .ibuiltins.basename import basename
from .ibuiltins.chunk_array import chunk_array
from .ibuiltins.create_folder import create_folder
from .ibuiltins.datetime_from_string import datetime_from_string
from .ibuiltins.datetime_now import datetime_now
from .ibuiltins.dict_first import dict_first
from .ibuiltins.dirname import dirname
from .ibuiltins.exit_if_empty import exit_if_empty
from .ibuiltins.filter_empty import filter_empty
from .ibuiltins.get_class_method import get_class_method
from .ibuiltins.get_filemtime import get_filemtime
from .ibuiltins.get_filesize import get_filesize
from .ibuiltins.is_empty import is_empty
from .ibuiltins.is_iterable import is_iterable
from .ibuiltins.is_valid_url import is_valid_url
from .ibuiltins.password_generator import password_generator
from .ibuiltins.random_bool import random_bool
from .ibuiltins.set_timeout import set_timeout
from .ibuiltins.sets_ordered import sets_ordered
from .ibuiltins.str_cmp import str_cmp
from .ibuiltins.to_str import to_str
from .iconsole.choices import choices
from .iconsole.console_run import console_run
from .iconsole.input_char import input_char
from .iconsole.log import log
from .iconsole.print_colorize import print_colorize
from .iconsole.print_dir import print_dir
from .iconsole.print_log import print_log
from .iconsole.print_to_last_line import print_to_last_line
from .iconsole.text_colorize import text_colorize
from .idjango.APIMixinView import APIMixinView
from .iengineering.PintUreg import PintUreg
from .iengineering.PintUregQuantity import PintUregQuantity
from .iengineering.batch_calculate import batch_calculate
from .iengineering.batchmaker import batchmaker
from .iengineering.calculate import calculate
from .iflow.auto_reload import auto_reload
from .iflow.github_pull import github_pull
from .iflow.github_push import github_push
from .iflow.github_user import github_user
from .iflow.pip_freeze_without_version import pip_freeze_without_version
from .iflow.poetry_publish import poetry_publish
from .iflow.poetry_update_version import poetry_update_version
from .ifunctions.iargv import iargv
from .ifunctions.idumps import idumps
from .ifunctions.idumps_html import idumps_html
from .ifunctions.ienv import ienv
from .ifunctions.iexec import iexec
from .ifunctions.ijoin import ijoin
from .ifunctions.iloads import iloads
from .ifunctions.iloads_html import iloads_html
from .ifunctions.iopen import iopen
from .ifunctions.iprint import iprint
from .ifunctions.irange import irange
from .ifunctions.ireplace import ireplace
from .ifunctions.iscandir import iscandir
from .ifunctions.isplit import isplit


# is_empty()
__empty_list__ = [None, False, 0, -0]
__empty_list__ += ["0", "", "-0", "\n", "\t"]
__empty_list__ += [set(), dict(), list(), tuple()]


def is_empty(variable, empty=__empty_list__):
    """
    Mengecek apakah variable setara dengan nilai kosong pada empty.

    Pengecekan nilai yang setara menggunakan simbol '==', sedangkan untuk
    pengecekan lokasi memory yang sama menggunakan keyword 'is'

    ```python
    print(is_empty("teks"))
    print(is_empty(True))
    print(is_empty(False))
    print(is_empty(None))
    print(is_empty(0))
    print(is_empty([]))
    ```
    """
    for e in empty:
        if variable == e:
            return True
    return False



import re


def ireplace(
    string: str,
    replacements: dict,
    flags=re.DOTALL | re.MULTILINE | re.IGNORECASE,
):
    """
    STRing TRanslate mengubah string menggunakan kamus dari dict.
    Replacement dapat berupa text biasa ataupun regex pattern.
    Apabila replacement berupa regex, gunakan raw string `r"..."`
    Untuk regex capturing gunakan `(...)`, dan untuk mengaksesnya
    gunakan `\\1`, `\\2`, .., dst.

    ```python
    text = 'aku ini mau ke sini'
    replacements = {
        "sini": "situ",
        r"(ini)": r"itu dan \\1",
    }
    print(ireplace(text, replacements))
    ```
    """
    for i, v in replacements.items():
        string = re.sub(i, v, string, flags=flags)
    return string



def str_cmp(t1, t2):
    """
    Membandingakan string secara incase-sensitive menggunakan lower().
    Lebih cepat dibandingkan upper(), casefold(), re.fullmatch(), len().
    perbandingan ini sangat cepat, tetapi pemanggilan fungsi ini membutuhkan
    overhead yg besar.

    ```python
    print(str_cmp('teks1', 'Teks1'))
    ```
    """
    return t1.lower() == t2.lower()
